#include <assert.h>

#include "shared/physics/2d/2d_body_hull.h"
#include "shared/physics/2d/rigid/2d_rigid_sim.h"
#include "shared/physics/2d/rigid/2d_rigid_drag_force.h"
#include "shared/physics/2d/rigid/2d_rigid_gravity_force.h"
#include "shared/physics/2d/rigid/2d_rigid_spring_force.h"
using namespace ajb;
using namespace Physics::D2;
using namespace Physics::D2::Rigid;

// misc
#include "utils.h"
#include <vector>
using namespace std;

// media lib
#include "music.h"
#include "zsound.h"
#include "bitmap.h"

#include <stdlib.h>  // For memcpy
#include "web_editor.h"
#include "example.h"

Auto_Array <Web_Anchor *> web_anchors;
Auto_Array <Web_Strand *> web_strands;

int selected_anchor = -1;

void select_anchor(int index) {
    selected_anchor = index;
}

void delete_any_strand(Web_Anchor *anchor) {
    int i;
    for (i = 0; i < web_strands.items; i++) {
        Web_Strand *strand = web_strands[i];

        bool should_delete = false;
        if (strand->anchor0 == anchor) should_delete = true;
        if (strand->anchor1 == anchor) should_delete = true;
        
        if (should_delete) {
            web_strands[i] = web_strands[web_strands.items - 1];
            web_strands.items--;
            delete strand;
            i--;
        }
    }
}

void delete_anchor() {
    if (web_anchors.items == 0) return;

    int index = web_anchors.items - 1;
    Web_Anchor *anchor = web_anchors[index];
    if (selected_anchor == index) selected_anchor = -1;

    delete_any_strand(anchor);
    web_anchors.items--;
}

void connect_to_selected(int index) {
    if (selected_anchor == -1) return;

    if (index == selected_anchor) return;  // Don't connect a node to itself!

    Web_Anchor *anchor0 = web_anchors[selected_anchor];
    Web_Anchor *anchor1 = web_anchors[index];

    Web_Strand *strand = new Web_Strand;
    strand->anchor0 = anchor0;
    strand->anchor1 = anchor1;

    web_strands.add(strand);
}

void add_anchor(Vec2 position, float radius) {
    Web_Anchor *anchor = new Web_Anchor;
    anchor->position = position;
    anchor->rb.set_body(make_Body_Sphere(radius));
    anchor->index = web_anchors.items;
    anchor->tied_to_background = false;

    web_anchors.add(anchor);
}

int get_anchor() {
    extern Vec2 world_mouse;
    Vec2 position = world_mouse;

    float distance_min = FLT_MAX;
    float distance_max = WindowWidth * 0.05f;
    int best_match = -1;

    int i;
    for (i = 0; i < web_anchors.items; i++) {
        Web_Anchor *anchor = web_anchors[i];
        float dist = (anchor->position - position).len();
        if (dist > distance_max) continue;

        if (dist < distance_min) {
            distance_min = dist;
            best_match = i;
        }
    }

    return best_match;
}


int get_anchor_index(Web_Anchor *anchor) {
    return anchor->index;
}

void save_web() {
    FILE *f = fopen("saved.web", "wt");
    
    fprintf(f, "%d\n", web_anchors.items);

    int i;
    for (i = 0; i < web_anchors.items; i++) {
        Web_Anchor *anchor = web_anchors[i];
        fprintf(f, "   %f %f %d %d\n", 
                anchor->position.x(), anchor->position.y(), anchor->index,
                (int)anchor->tied_to_background);
    }


    fprintf(f, "%d\n", web_strands.items);
    for (i = 0; i < web_strands.items; i++) {
        Web_Strand *strand = web_strands[i];
        fprintf(f, " %d %d\n", get_anchor_index(strand->anchor0),
                get_anchor_index(strand->anchor1));
    }

    fclose(f);
}

void load_web() {
    FILE *f = fopen("saved.web", "rt");
    if (!f) return;

    int num_anchors;
    int success = fscanf(f, "%d\n", &num_anchors);
    assert(success == 1);

    int i;
    for (i = 0; i < num_anchors; i++) {
        float x, y;
        int index, tied;
        success = fscanf(f, "   %f %f %d %d\n", &x, &y, &index, &tied);
        
        Web_Anchor *anchor = new Web_Anchor;
        anchor->position.set_x(x);
        anchor->position.set_y(y);
        anchor->rb.set_body(make_Body_Sphere(ANCHOR_RADIUS));
        anchor->index = index;
        anchor->tied_to_background = tied;

        web_anchors.add(anchor);

        assert(index == i);
    }

    int num_strands;
    success = fscanf(f, "%d\n", &num_strands);
    assert(success == 1);
    for (i = 0; i < num_strands; i++) {
        int n0, n1;
        success = fscanf(f, " %d %d\n", &n0, &n1);

        if (n0 >= web_anchors.items) continue;
        if (n1 >= web_anchors.items) continue;

        Web_Strand *strand = new Web_Strand;
        strand->anchor0 = web_anchors[n0];
        strand->anchor1 = web_anchors[n1];

        web_strands.add(strand);
    }

}

